@@ -24,9 +24,12 @@ using namespace CPPUNIT_NS;
 // compile-time checks for binary conversion
 static_assert(toSynchsafeInt(255) == 383, "toSynchsafeInt()");
 static_assert(toNormalInt(383) == 255, "toNormalInt()");
-static_assert(swapOrder(static_cast<std::uint16_t>(0xABCD)) == 0xCDAB, "swapOrder(uint16)");
-static_assert(swapOrder(static_cast<std::uint32_t>(0xABCDEF12)) == 0x12EFCDAB, "swapOrder(uint32)");
-static_assert(swapOrder(static_cast<std::uint64_t>(0xABCDEF1234567890)) == 0x9078563412EFCDAB, "swapOrder(uint64)");
+static_assert(swapOrder(static_cast<std::uint16_t>(0xABCD)) == 0xCDAB, "swapOrder(std::uint16_t)");
+static_assert(swapOrder(static_cast<std::uint32_t>(0xABCDEF12u)) == 0x12EFCDABu, "swapOrder(std::uint32_t)");
+static_assert(swapOrder(static_cast<std::uint64_t>(0xABCDEF1234567890ul)) == 0x9078563412EFCDABul, "swapOrder(std::uint64_t)");
+static_assert(swapOrder(static_cast<std::int16_t>(0xABCD)) == static_cast<std::int16_t>(0xCDAB), "swapOrder(std::int16_t)");
+static_assert(swapOrder(static_cast<std::int32_t>(0xABCDEF12)) == 0x12EFCDAB, "swapOrder(std::int32_t)");
+static_assert(swapOrder(static_cast<std::int64_t>(0xABCDEF1234567890l)) == static_cast<std::int64_t>(0x9078563412EFCDABl), "swapOrder(std::int64_t)");
 
 /*!
  * \brief The ConversionTests class tests classes and functions provided by the files inside the conversion directory.
@@ -116,13 +119,13 @@ void ConversionTests::testConversion(
     stringstream msg;
     msg << message << '(' << hex << '0' << 'x' << random << ')';
     vice(random, m_buff);
-    CPPUNIT_ASSERT_MESSAGE(msg.str(), versa(m_buff) == random);
+    CPPUNIT_ASSERT_EQUAL_MESSAGE(msg.str(), random, versa(m_buff));
 }
 
 #define TEST_TYPE(endianness, function) decltype(endianness::function(m_buff))
 
 #define TEST_CONVERSION(function, endianness)                                                                                                        \
-    testConversion<TEST_TYPE(endianness, function)>("testing " #function,                                                                            \
+    testConversion<TEST_TYPE(endianness, function)>("testing " #endianness "::" #function,                                                           \
         static_cast<void (*)(TEST_TYPE(endianness, function), char *)>(&endianness::getBytes), endianness::function,                                 \
         numeric_limits<TEST_TYPE(endianness, function)>::min(), numeric_limits<TEST_TYPE(endianness, function)>::max())
 
@@ -131,8 +134,8 @@ void ConversionTests::testConversion(
 #define TEST_LE_CONVERSION(function) TEST_CONVERSION(function, LE)
 
 #define TEST_CUSTOM_CONVERSION(vice, versa, endianness, min, max)                                                                                    \
-    testConversion<TEST_TYPE(endianness, versa)>(                                                                                                    \
-        "testing " #versa, static_cast<void (*)(TEST_TYPE(endianness, versa), char *)>(&endianness::vice), endianness::versa, min, max)
+    testConversion<TEST_TYPE(endianness, versa)>("testing " #versa " (" #endianness ")",                                                             \
+        static_cast<void (*)(TEST_TYPE(endianness, versa), char *)>(&endianness::vice), endianness::versa, min, max)
 
 /*!
  * \brief Tests most important binary conversions.
@@ -209,6 +212,7 @@ void assertEqual(const char *message, const std::uint8_t *expectedValues, size_t
  */
 void ConversionTests::testStringEncodingConversions()
 {
+#ifndef CPP_UTILITIES_NO_ICONV
     // define test string "ABCD" for the different encodings
     const std::uint8_t simpleString[] = { 'A', 'B', 'C', 'D' };
     const std::uint16_t simpleUtf16LEString[] = { 0x0041, 0x0042, 0x0043, 0x0044 };
@@ -239,6 +243,7 @@ void ConversionTests::testStringEncodingConversions()
     assertEqual("UTF-8 to UFT-16BE", reinterpret_cast<const std::uint8_t *>(BE_STR_FOR_ENDIANNESS(utf16)), 10,
         convertUtf8ToUtf16BE(reinterpret_cast<const char *>(utf8String), 6));
     CPPUNIT_ASSERT_THROW(convertString("invalid charset", "UTF-8", "foo", 3, 1.0f), ConversionException);
+#endif
 }
 
 /*!
@@ -256,16 +261,16 @@ void ConversionTests::testStringConversions()
         auto signedRandom = randomDistSigned(m_randomEngine);
         auto unsignedRandom = randomDistUnsigned(m_randomEngine);
         for (const auto base : initializer_list<std::uint8_t>{ 2, 8, 10, 16 }) {
-            const auto asString = numberToString<std::uint64_t, string>(unsignedRandom, static_cast<string::value_type>(base));
+            const auto asString = numberToString<std::uint64_t, string>(unsignedRandom, base);
             const auto asWideString = numberToString<std::uint64_t, wstring>(unsignedRandom, base);
-            CPPUNIT_ASSERT_EQUAL_MESSAGE(stringMsg, unsignedRandom, stringToNumber<std::uint64_t>(asString, static_cast<string::value_type>(base)));
+            CPPUNIT_ASSERT_EQUAL_MESSAGE(stringMsg, unsignedRandom, stringToNumber<std::uint64_t>(asString, base));
             CPPUNIT_ASSERT_EQUAL_MESSAGE(wideStringMsg, unsignedRandom, stringToNumber<std::uint64_t>(asWideString, base));
             CPPUNIT_ASSERT_EQUAL_MESSAGE(bufferMsg, unsignedRandom, bufferToNumber<std::uint64_t>(asString.data(), asString.size(), base));
         }
         for (const auto base : initializer_list<std::uint8_t>{ 10 }) {
             const auto asString = numberToString<std::int64_t, string>(signedRandom, static_cast<string::value_type>(base));
             const auto asWideString = numberToString<std::int64_t, wstring>(signedRandom, base);
-            CPPUNIT_ASSERT_EQUAL_MESSAGE(stringMsg, signedRandom, stringToNumber<std::int64_t>(asString, static_cast<string::value_type>(base)));
+            CPPUNIT_ASSERT_EQUAL_MESSAGE(stringMsg, signedRandom, stringToNumber<std::int64_t>(asString, base));
             CPPUNIT_ASSERT_EQUAL_MESSAGE(wideStringMsg, signedRandom, stringToNumber<std::int64_t>(asWideString, base));
             CPPUNIT_ASSERT_EQUAL_MESSAGE(bufferMsg, signedRandom, bufferToNumber<std::int64_t>(asString.data(), asString.size(), base));
         }
@@ -283,18 +288,37 @@ void ConversionTests::testStringConversions()
     CPPUNIT_ASSERT_EQUAL(23u, stringToNumber<std::uint32_t>("  023"s));
     CPPUNIT_ASSERT_EQUAL(23u, bufferToNumber<std::uint32_t>("  023", 5));
     CPPUNIT_ASSERT_EQUAL(255u, stringToNumber<std::uint32_t>("fF", 16));
-    CPPUNIT_ASSERT_THROW(stringToNumber<std::uint32_t>("fF", 15), ConversionException);
-    CPPUNIT_ASSERT_THROW(stringToNumber<std::uint32_t>("(", 15), ConversionException);
+    CPPUNIT_ASSERT_THROW_MESSAGE("character out of range", stringToNumber<std::uint32_t>("fF", 15), ConversionException);
+    CPPUNIT_ASSERT_THROW_MESSAGE("invalid character", stringToNumber<std::uint32_t>("(", 15), ConversionException);
+#ifdef __GNUC__ // overflow detection only supported on GCC and Clang
+    CPPUNIT_ASSERT_THROW_MESSAGE("overflow", stringToNumber<std::uint32_t>("100000000", 16), ConversionException);
+    CPPUNIT_ASSERT_THROW_MESSAGE("underflow", stringToNumber<std::int32_t>("-80000001", 16), ConversionException);
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("positive limit", 0xFFFFFFFFu, stringToNumber<std::uint32_t>("FFFFFFFF", 16));
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("negative limit", -2147483647, stringToNumber<std::int32_t>("-2147483647", 10));
+#endif
+
+    // stringToNumber() / numberToString() with floating point numbers
+    CPPUNIT_ASSERT_EQUAL(1.5f, stringToNumber<float>(numberToString(1.5f)));
+    CPPUNIT_ASSERT_EQUAL(1.5, stringToNumber<double>(numberToString(1.5)));
+    CPPUNIT_ASSERT_EQUAL(-10.25, stringToNumber<double>("-10.25"));
 
     // interpretIntegerAsString()
     CPPUNIT_ASSERT_EQUAL("TEST"s, interpretIntegerAsString<std::uint32_t>(0x54455354));
 
     // splitString() / joinStrings()
+    CPPUNIT_ASSERT_EQUAL_MESSAGE("empty string", vector<string>({ string() }), splitString<vector<string>>(string(), ","));
+    CPPUNIT_ASSERT_EQUAL_MESSAGE(
+        "empty string (simple)", vector<string_view>({ string_view() }), splitStringSimple<vector<string_view>>(string_view(), ","));
     vector<string> splitTestExpected({ "1", "2,3" });
     vector<string> splitTestActual = splitString<vector<string>>("1,2,3"s, ","s, EmptyPartsTreat::Keep, 2);
     CPPUNIT_ASSERT_EQUAL(splitTestExpected, splitTestActual);
     splitTestActual = splitStringSimple<vector<string>>("1,2,3"s, ","s, 2);
     CPPUNIT_ASSERT_EQUAL(splitTestExpected, splitTestActual);
+    splitTestExpected = { "12", "34", "56", "" };
+    splitTestActual = splitString<vector<string>>("12,34,56,"s, ","s);
+    CPPUNIT_ASSERT_EQUAL(splitTestExpected, splitTestActual);
+    splitTestActual = splitStringSimple<vector<string>>("12,34,56,"s, ","s);
+    CPPUNIT_ASSERT_EQUAL(splitTestExpected, splitTestActual);
     splitTestExpected = { "1", "2,3", "4,,5" };
     splitTestActual = splitString<vector<string>>("1,2,,3,4,,5"s, ","s, EmptyPartsTreat::Merge, 3);
     CPPUNIT_ASSERT_EQUAL(splitTestExpected, splitTestActual);
@@ -344,10 +368,10 @@ void ConversionTests::testStringConversions()
     CPPUNIT_ASSERT_EQUAL("foo"s, truncateTest);
 
     // encodeBase64() / decodeBase64() with random data
-    uniform_int_distribution<std::uint8_t> randomDistChar;
+    std::uniform_int_distribution<int> randomDistChar;
     std::uint8_t originalBase64Data[4047];
     for (std::uint8_t &c : originalBase64Data) {
-        c = randomDistChar(m_randomEngine);
+        c = static_cast<std::uint8_t>(randomDistChar(m_randomEngine) & 0xFF);
     }
     auto encodedBase64Data = encodeBase64(originalBase64Data, sizeof(originalBase64Data));
     auto decodedBase64Data = decodeBase64(encodedBase64Data.data(), static_cast<std::uint32_t>(encodedBase64Data.size()));
@@ -394,16 +418,10 @@ struct StringThatDoesNotLikeToBeCopiedOrMoved : public std::string {
         : std::string(value)
     {
     }
-    [[noreturn]] StringThatDoesNotLikeToBeCopiedOrMoved(const StringThatDoesNotLikeToBeCopiedOrMoved &other)
-        : std::string(other)
-    {
-        CPPUNIT_FAIL("attempt to copy string: " + other);
-    }
-    [[noreturn]] StringThatDoesNotLikeToBeCopiedOrMoved(StringThatDoesNotLikeToBeCopiedOrMoved &&other)
-        : std::string(std::move(other))
-    {
-        CPPUNIT_FAIL("attempt to move string: " + other);
-    }
+    // delete copy and move c'tor so if argsToString() would invoke one it would be a compile time error
+    // caveat: argsToString() might still construct an std::string() unnoticed.
+    [[noreturn]] StringThatDoesNotLikeToBeCopiedOrMoved(const StringThatDoesNotLikeToBeCopiedOrMoved &other) = delete;
+    [[noreturn]] StringThatDoesNotLikeToBeCopiedOrMoved(StringThatDoesNotLikeToBeCopiedOrMoved &&other) = delete;
 };
 
 /// \endcond
@@ -452,8 +470,8 @@ void ConversionTests::testStringBuilder()
     CPPUNIT_ASSERT_EQUAL_MESSAGE("using string_view", "foobar123"s, "foo"sv % "bar"sv + 123);
 
     // check that for the internal tuple construction no copies are made
-    StringThatDoesNotLikeToBeCopiedOrMoved str(" happen ");
-    const StringThatDoesNotLikeToBeCopiedOrMoved str2("for this");
+    auto str = StringThatDoesNotLikeToBeCopiedOrMoved(" happen ");
+    const auto str2 = StringThatDoesNotLikeToBeCopiedOrMoved("for this");
     CPPUNIT_ASSERT_EQUAL("no copy/move should happen for this!"s,
         argsToString(StringThatDoesNotLikeToBeCopiedOrMoved("no copy/move should"), str, str2, StringThatDoesNotLikeToBeCopiedOrMoved("!")));
 }
